package uk.co.twey.uni.dooom;

/**
 * Represents an abstract command-processor, responsible for reading
 * and executing commands.
 */
public abstract class CommandProcessorOfDooom {
    /**
     * Execute a line.
     *
     * @param client the client attempting to execute the command.
     * @param line the command line sent.
     */
    public void receiveLine(ClientOfDooom client, String line) {
        String[] args = line.split(" ", 2);

        receive(client, args[0],
                java.util.Arrays.<String>copyOfRange(args, 1,
                                                     args.length));
    }

    /**
     * Attempt to execute a command.  Calls the command if a matching
     * {@link CommandOfDooom} can be found; otherwise, calls onUnknown.
     *
     * @param client the client attempting to execute the command.
     * @param cmdName the name of the command being executed.
     * @param args the arguments passed to the command.
     */
    public void receive(ClientOfDooom client, String cmdName,
                        String... args) {
        CommandOfDooom cmd = commands.get(cmdName);

        if (cmd == null)
            onUnknown(client, cmdName, args);
        else
            cmd.checkedExecute(client, cmdName, args);
    }

    /**
     * Executed when a command is requested that cannot be found in
     * the command table.
     *
     * @param client the client attempting to execute the command.
     * @param cmd the name of the command being executed.
     * @param args the arguments passed to the command.
     */
    protected abstract void onUnknown(ClientOfDooom client, String cmd,
                                      String... args);

    /**
     * The command table.  Add a pair of &lt;String, {@link
     * CommandOfDooom}&gt; to this map to associate the action with
     * the command.
     */
    protected java.util.Map<String, CommandOfDooom> commands;
}
