package uk.co.twey.uni.dooom;

/**
 * Superclass of exceptions thrown when something happens on trying to enter a
 * place.
 */
public abstract class EntryException extends Exception {
    /**
     * The player who attempted to enter the place.
     */
    protected PlayerOfDooom player;
    public PlayerOfDooom getPlayer() { return player; }

    /**
     * The place the player attempted to enter.
     */
    protected PlaceOfDooom place;
    public PlaceOfDooom getPlace() { return place; }

    /**
     * Create a new EntryException.
     *
     * @param player the player who attempted to enter the place.
     * @param place the place into which entry was attempted.
     * @param message a message describing the event.
     */
    public EntryException(PlayerOfDooom player, PlaceOfDooom place,
                          String message) {
        super(message);

        this.player = player;
        this.place = place;
    }
}