package uk.co.twey.uni.dooom;

import java.util.*;

/**
 * Represents a floor tile, with or without items or players.
 */
public class FloorOfDooom extends PlaceOfDooom implements HolderOfDooom {
    /** The items lying on this floor tile. */
    protected Set<ObjectOfDooom> items = new HashSet<ObjectOfDooom>();
    public Set<ObjectOfDooom> getItems() { return items; }

    /**
     * Create a new FloorOfDooom.
     *
     * @param x the absolute x position of the tile.
     * @param y the absolute y position of the tile.
     * @param things the objects stored on this tile (including players).
     */
    public FloorOfDooom(int x, int y, ObjectOfDooom... things) {
        super(x, y);

        for (ObjectOfDooom thing : things)
            try {
                thing.giveTo(this);
            } catch (PossessionException e) {
            }
    }

    public void take(ObjectOfDooom thing) {
        items.add(thing);
    }

    public void drop(ObjectOfDooom thing) {
        items.remove(thing);
    }

    public void onEnter(PlayerOfDooom p) throws EntryException {
        for (ObjectOfDooom thing : getItems())
            if (thing instanceof PlayerOfDooom && p != thing)
                throw new PlaceOccupiedException(p, this, thing);
    }

    /**
     * The icon to use when no object on the floor overrides it.
     * <p>
     * The base icon for floor is a dot (.).
     */
    public char getBaseIcon() { return '.'; }

    /**
     * {@inheritDoc}
     * <p>
     * Floor can contain multiple objects, so the icon displayed is
     * chosen according to the highest-precedence item on the floor.
     *
     * @return {@inheritDoc}
     */
    public char getIcon() {
        ObjectOfDooom decider = null;

        for (ObjectOfDooom item : items)
            if (decider == null
                    || item.getPrecedence() > decider.getPrecedence())
                decider = item;

        return decider == null ? getBaseIcon() : decider.getIcon();
    }

    /**
     * As getIcon(), but disregarding one player.
     */
    public char getIconNoPlayer() {
        ObjectOfDooom decider = null;

        boolean playerSeen = false;

        for (ObjectOfDooom item : items)
            if ((!(item instanceof PlayerOfDooom)
                 || playerSeen
                 || !(playerSeen = true))
                && (decider == null
                    || item.getPrecedence() > decider.getPrecedence()))
                decider = item;

        return decider == null ? getBaseIcon() : decider.getIcon();
    }
}