package uk.co.twey.uni.dooom;

/**
 * Represents a quantity of gold.
 */
public class GoldOfDooom extends ObjectOfDooom {
    /** How much gold is stored in this quantity. */
    protected int amount;
    public int getAmount() { return amount; }

    /** {@inheritDoc} <p> The icon for gold is G. */
    public char getIcon() { return 'G'; }
    /** {@inheritDoc} <p> Gold has a precedence of 1. */
    public int getPrecedence() { return 1; }
    /** {@inheritDoc} <p> Example: "3 gold". */
    public String getName() { return "" + amount + " gold"; }

    /**
     * Create a new GoldOfDooom with a quantity of 1.
     */
    public GoldOfDooom() { this(1); }

    /**
     * Create a new GoldOfDooom.
     *
     * @param amount the value of the gold.
     */
    public GoldOfDooom(int amount) {
        this.amount = amount;
    }

    /**
     * {@inheritDoc}
     * <p>
     * Destroys the gold and adds its value to the player's gold count.
     */
    public void onTake(HolderOfDooom holder) throws PossessionException {
        if (holder instanceof PlayerOfDooom) {
            PlayerOfDooom player = (PlayerOfDooom) holder;
            player.addGold(getAmount());
            player.tell("TREASUREMOD " + getAmount());
            holder.drop(this);
        }
    }
}