package uk.co.twey.uni.dooom;

/** Represents any object that can be moved around the map, including
    players, items, et cetera. */
public abstract class ObjectOfDooom {
    /**
     * Get the icon used to represent this object when displaying the map.
     */
    public abstract char getIcon();

    /**
     * Get the precedence of this object, used for calculating which
     * object should be displayed on a particular place.
     */
    public abstract int getPrecedence();

    /**
     * Get a human-readable name for this object.
     */
    public abstract String getName();

    /** The object or place that currently holds this object. */
    protected HolderOfDooom holder;
    public HolderOfDooom getHolder() { return holder; }

    /**
     * Remove the object from the possession of its current holder and
     * transfer it to a new holder.
     *
     * @param holder the beneficiary of the transfer.
     *
     * @throws PossessionException when the specified holder cannot
     * hold this item (e.g. players cannot pick up other players).
     */
    public void giveTo(HolderOfDooom holder) throws PossessionException {
        onTake(holder);
        if (this.holder != null) this.holder.drop(this);
        holder.take(this);
        this.holder = holder;
    }

    /**
     * Event handler called before transferring the object to its new
     * owner. Throw a PossessionException here to cancel the transfer.
     * <p>
     * The default handler does nothing.
     *
     * @param holder the potential new owner.
     *
     * @throws PossessionException when the specified holder cannot
     * hold this item (e.g. players cannot pick up other players).
     */
    public void onTake(HolderOfDooom holder) throws PossessionException { }
}