package uk.co.twey.uni.dooom;

/** An abstract class representing any type of place (i.e. an integral
 * part of the map itself, as opposed to an object on the map). */
public abstract class PlaceOfDooom {
    /** The absolute x and y coördinates of this place on the map. */
    protected int x = -1, y = -1;
    public int getX() { return x; };
    public void setX(int x) { this.x = x; };
    public int getY() { return y; };
    public void setY(int y) { this.y = y; };

    /**
     * Create a new PlaceOfDooom.
     *
     * @param x the absolute x-coördinate of this place on the map.
     * @param y the absolute y-coördinate of this place on the map.
     */
    public PlaceOfDooom(int x, int y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Find the distance between this place and another.
     *
     * @param other the other end-point.
     *
     * @return the distance as a float.
     */
    public float distanceFrom(PlaceOfDooom other) {
        return (float) Math.sqrt(Math.pow(other.x - x, 2)
                                 + Math.pow(other.y - y, 2));
    }

    /**
     * Determine the single-character icon that should be used to
     * represent this place when displaying the map to the player.
     */
    public abstract char getIcon();

    /**
     * Return whether or not a player may enter this place.  Defaults
     * to true.
     */
    public boolean isPassable() { return true; };

    /**
     * An event handler called after a player enters the place.
     *
     * @param p the player attempting to enter.
     */
    public void onEnter(PlayerOfDooom p) throws EntryException { };
}