package uk.co.twey.uni.dooom.client;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.co.twey.uni.dooom.ui.*;

/** The main window of the client UI. */
public class ClientUIWindow extends JFrame {
    static final long serialVersionUID = -2093422857482551613L;

    /** The outward-facing connection to the server.  User-initiated
     * commands will be written here. */
    protected PrintWriter out;

    /** The {@link MapWidget} used to display the map. */
    protected MapWidget mapWidget;
    /** The {@link LogWidget} used to display the log. */
    protected LogWidget logWidget;
    // protected CommandWidget commandWidget;
    /** The {@link StatusWidget} used to display player/game
     * statistics. */
    protected StatusWidget statusWidget;

    /**
     * Create a new ClientUIWindow.
     *
     * @param out the outward-facing connection to the server.
     * User-initiated commands will be written here.
     */
    public ClientUIWindow(PrintWriter out) {
        super("ClientOfDooom");

        this.out = out;

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(500, 300);

        setContentPane(new JPanel(new BorderLayout()));

        JPanel westPane = new JPanel(new GridLayout(2, 1));
        westPane.add(statusWidget = new StatusWidget());
        westPane.add(new HelpWidget("control"));

        add(mapWidget = new MapWidget("XXXXXXX\nXXXXXXX\nXXXXXXX\n"
                                      + "XXXXXXX\nXXXXXXX\nXXXXXXX\nXXXXXXX"),
            BorderLayout.CENTER);
        add(logWidget = new LogWidget(), BorderLayout.EAST);
        // add(commandWidget = new CommandWidget(this, out), BorderLayout.SOUTH);
        add(westPane, BorderLayout.WEST);

        addKeyBindings();

        pack();
        setVisible(true);
        // commandWidget.requestFocusInWindow();
    }

    /**
     * Helper method to add key bindings to the UI.
     */
    protected void addKeyBindings() {
        InputMap iMap = ((JPanel) getContentPane()).getInputMap(
                          JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        ActionMap aMap = ((JPanel) getContentPane()).getActionMap();

        iMap.put(KeyStroke.getKeyStroke("O"), "moveWest");
        iMap.put(KeyStroke.getKeyStroke("S"), "moveWest");
        iMap.put(KeyStroke.getKeyStroke("I"), "moveEast");
        iMap.put(KeyStroke.getKeyStroke("D"), "moveEast");
        iMap.put(KeyStroke.getKeyStroke("E"), "moveNorth");
        iMap.put(KeyStroke.getKeyStroke("W"), "moveNorth");
        iMap.put(KeyStroke.getKeyStroke("U"), "moveSouth");
        iMap.put(KeyStroke.getKeyStroke("S"), "moveSouth");
        iMap.put(KeyStroke.getKeyStroke("P"), "pickup");
        iMap.put(KeyStroke.getKeyStroke("Q"), "pickup");
        iMap.put(KeyStroke.getKeyStroke("SPACE"), "endTurn");

        aMap.put("moveWest", new CommandHandler("MOVE W"));
        aMap.put("moveEast", new CommandHandler("MOVE E"));
        aMap.put("moveSouth", new CommandHandler("MOVE S"));
        aMap.put("moveNorth", new CommandHandler("MOVE N"));
        aMap.put("pickup", new CommandHandler("PICKUP"));
        aMap.put("endTurn", new CommandHandler("ENDTURN"));
    }

    /**
     * Helper class to handle an ActionEvent by sending a command to
     * the server.
     */
    protected class CommandHandler extends AbstractAction {
        protected String cmd;

        public CommandHandler(String cmd) {
            this.cmd = cmd;
        }

        public void actionPerformed(ActionEvent ev) {
            logWidget.append("<< " + cmd);
            out.println(cmd);
        }
    }

    /**
     * Append a message to the log.
     *
     * @param msg the message to append.
     */
    public void appendLog(String msg) {
        logWidget.append(msg);
    }

    /**
     * Update the map widget.
     *
     * @param plan the string representation of the new map to
     * display.
     */
    public void updateMap(String plan) {
        mapWidget.update(plan, true);
    }

    /**
     * Update the amount of gold held by the player.
     *
     * @param gold the new amount of gold.
     */
    public void setGold(int gold) {
        statusWidget.setGold(gold);
    }

    /**
     * Update the amount of gold needed to win the game.
     *
     * @param goal the new goal.
     */
    public void setGoal(int goal) {
        statusWidget.setGoal(goal);
    }
}