package uk.co.twey.uni.dooom.client;

import java.io.*;
import java.net.*;

/**
 * A launcher for the client.
 */
public class Main {
    /**
     * Entry point for the client.
     *
     * @param args the arguments passed to the program.  Acceptable
     * arguments are:
     * <ul>
     *   <li>-h [host]: the host to which to connect
     *   <li>-p [port]: the port to use
     * </ul>
     */
    public static void main(String[] args) {
        Socket sock = null;
        ClientUI ui = null;
        BufferedReader in = null;
        PrintWriter out = null;

        String sHost = "127.0.0.1", sPort = "43200", sName = null;
        
        for (int i = 0, n = args.length; i < n; ++i)
            if ("-h".equals(args[i]) && i < n + 1)
                sHost = args[++i];
            else if ("-p".equals(args[i]) && i < n + 1)
                sPort = args[++i];
            else if ("-n".equals(args[i]) && i < n + 1)
                sName = args[++i];
            else
                printUsage();

        try {
            sock = new Socket(sHost, Integer.parseInt(sPort));
            ui = new ClientUI(out = new PrintWriter(sock.getOutputStream(),
                                                    true));
            in = new BufferedReader(
                     new InputStreamReader(sock.getInputStream()));
        } catch (NumberFormatException e) {
            printUsage();
        } catch (UnknownHostException e) {
            System.err.println("Couldn't look up host \"" + sHost + "\".");
            System.exit(2);
        } catch (IOException e) {
            System.err.println("Couldn't connect to host \"" + sHost
                               + "\" on port " + sPort + ".");
            System.exit(3);
        }

        if (sName != null)
            out.println("HELLO " + sName);
        else
            out.println("HELLO");

        String line;

        try {
            while ((line = in.readLine()) != null)
                ui.receiveLine(line, in);
            ui.receiveLine(null, null);
        } catch (IOException e) {
            ui.receiveLine(null, null);
        }
    }

    /**
     * Print out the usage for this program.
     */
    protected static void printUsage() {
        System.err.println("Usage: java " + Main.class.getName()
                           + " [-h host] [-p port] [-n name]");
        System.exit(1);
    }
}