package uk.co.twey.uni.dooom.server;

import uk.co.twey.uni.dooom.*;
import java.io.*;

/**
 * Launcher for the server.
 */
public class Main {
    /**
     * Entry point.
     *
     * @param args the arguments passed to the program; accepted arguments are:
     * <ul>
     *   <li>-p [port]: the port to which to bind</li>
     *   <li>-m [map]: the map to load</li>
     * </ul>
     */
    public static void main(String[] args) {
        String sPort = "43200";
        int port = 0;
        String sMap = "maps/default.map";
        MapOfDooom map = null;

        for (int i = 0, n = args.length; i < n; ++i)
            if ("-p".equals(args[i]) && i < n + 1)
                sPort = args[++i];
            else if ("-m".equals(args[i]) && i < n + 1)
                sMap = args[++i];
            else
                printUsage();

        try {
            port = Integer.parseInt(sPort);
            map = MapOfDooom.fromFile(new File(sMap));
        } catch (NumberFormatException e) {
            printUsage();
        } catch (IOException e) {
            System.err.println("Could not read map file " + sMap);
            System.exit(9);
        } catch (MapFormatException e) {
            System.err.println("Map file " + sMap + " is not in the correct format.");
            System.exit(10);
        }

        try {
            
            Server server = (new Server(new GameOfDooom(map), port));
            ServerUI ui = new ServerUI(server);
            server.setUI(ui);
            server.run();
        } catch (IOException e) {
            System.err.println("Could not start server:\n" + e);
        }
    }

    /**
     * Print out the usage of the command.
     */
    protected static void printUsage() {
        System.err.println("Usage: java " + Main.class.getName() + " [-m <map>] [-p <port>]");
        System.exit(1);
    }
}