package uk.co.twey.uni.dooom.server;

import uk.co.twey.uni.dooom.*;

import java.net.*;
import java.io.*;

/**
 * Represents a running server.
 */
public class Server implements Runnable {
    /** The game being played. */
    protected GameOfDooom game;
    public GameOfDooom getGame() { return game; }

    /** The port on which to listen. */
    protected int port;
    public int getPort() { return port; }

    /** Whether the server is not listening for connections. */
    protected boolean stopped = false;

    /**
     * Stop listening for connections safely.
     */
    public synchronized void stop() {
        stopped = true; 
        try {
            sock.close();
        } catch (IOException e) {
            System.err.println("Server.stop(): Couldn't close socket:\n" + e);
        }
    }

    /**
     * Begin listening for connections.
     */
    public synchronized void start() throws IOException {
        stopped = false;

        sock = new ServerSocket(port);
    }

    /**
     * Begin listening for connections on a new port.
     *
     * @param port the port to use.
     */
    public synchronized void startWithPort(int port) throws IOException {
        this.port = port;

        start();
    }

    /** The {@link ServerUI} controlling this server instance. */
    protected ServerUI ui;
    public void setUI(ServerUI ui) { this.ui = ui; }

    /** The socket on which to listen. */
    protected ServerSocket sock;

    /**
     * Create and start a new Server.
     *
     * @param game the game to play.
     * @param port the port on which to listen.
     *
     * @throws IOException if the server could not be started.
     */
    public Server(GameOfDooom game, int port) throws IOException {
        this.game = game;
        this.port = port;

        start();
    }

    /**
     * Run the server.
     */
    public void run() {
        while (true)
            try {
                if (stopped)
                    Thread.sleep(500);
                else
                    new Thread(new ServerClient(this, ui, sock.accept())).start();
            } catch (IOException e) {
                // Probably just got the socket closed; no way to tell.
            } catch (InterruptedException e) {
                // Nothing to see here, move along.
            }
    }
}