package uk.co.twey.uni.dooom.server;

import java.awt.*;
import javax.swing.*;

import uk.co.twey.uni.dooom.*;
import uk.co.twey.uni.dooom.ui.*;

/**
 * The main window for the server UI.
 */
public class ServerUIWindow extends JFrame {
    static final long serialVersionUID = 3761618265413780553L;

    /** The {@link MapWidget} used to display the current map. */
    protected MapWidget mapWidget;
    /** The {@link ServerWidget} to control the server. */
    protected ServerWidget serverWidget;
    /** The {@link LogWidget} to display traffic. */
    protected LogWidget logWidget;

    /** The {@link Server} object. */
    protected Server server;

    /**
     * Create a new ServerUIWindow.
     * 
     * @param server the {@link Server} object.
     */
    public ServerUIWindow(Server server) {
        super("Server");

        this.server = server;

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(500, 300);

        setContentPane(new JPanel(new BorderLayout()));

        add(mapWidget = new MapWidget(server.getGame().getMap().getPlan()),
            BorderLayout.CENTER);
        add(serverWidget = new ServerWidget(server), BorderLayout.SOUTH);
        add(logWidget = new LogWidget(), BorderLayout.EAST);

        pack();
        setVisible(true);
    }

    /**
     * Append a message to the log.
     *
     * @param msg the message to append.
     */
    public void appendLog(String msg) {
        logWidget.append(msg);
    }

    /**
     * Update the map widget with the current server map.
     */
    public void updateMap() {
        mapWidget.update(server.getGame().getMap().getPlan());
    }
}