package uk.co.twey.uni.dooom.server;

import java.io.*;
import java.net.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A widget that displays information about and allows control of a
 * listening network server.
 */
public class ServerWidget extends JPanel {
    static final long serialVersionUID = -2522366002937528911L;

    /** The {@link Server} object to be controlled. */
    protected Server server;
    /** The current host. */
    protected String host;

    /** The button used to stop/start the server. */
    protected JButton stopButton;
    /** The field containing the port number to use. */
    protected JTextField portField;

    /**
     * Create a new ServerWidget.
     *
     * @param server the {@link Server} object to be controlled.
     */
    public ServerWidget(Server server) {
        this.server = server;
        try {
            this.host = InetAddress.getLocalHost().getCanonicalHostName();
        } catch (UnknownHostException e) {
            this.host = "0.0.0.0"; // Probably…
        }

        portField = new JTextField("" + server.getPort());
        portField.setEnabled(false);

        (stopButton = new JButton("Stop"))
            .addActionListener(new StopButtonListener(server, portField));

        add(new JLabel(host + " : "));
        add(portField);
        add(stopButton);
    }

    /**
     * A helper class to stop/start the server in response to
     * activation of the stop/start button.
     */
    protected static class StopButtonListener implements ActionListener {
        protected Server server;
        protected JTextField portField;
        protected boolean stopped = false;

        /**
         * Create a new StopButtonListener.
         *
         * @param server the {@link Server} object to be controlled.
         * @param portField the text entry containing the port number
         * to be used.
         */
        public StopButtonListener(Server server, JTextField portField) {
            this.server = server;
            this.portField = portField;
        }

        /**
         * Stop/start the server.
         */
        public void actionPerformed(ActionEvent ev) {
            if (stopped = !stopped) {
                server.stop();
                portField.setEnabled(true);
                ((JButton) ev.getSource()).setText("Start");
            } else {
                try {
                    server.startWithPort(Integer.parseInt(portField.getText()));
                    portField.setEnabled(false);
                    ((JButton) ev.getSource()).setText("Stop");
                } catch (NumberFormatException e) {
                    portField.setText("" + server.getPort());
                    stopped = true;
                } catch (IOException e) {
                    stopped = true;

                    System.err.println("Couldn't open port:\n" + e);
                }
            }
        }
    }
}