package uk.co.twey.uni.dooom.ui;

import uk.co.twey.uni.dooom.*;

import java.util.*;
import java.awt.*;
import javax.swing.*;

/**
 * A generic Swing map-displaying widget, used by both server and
 * client GUIs.
 */
public class MapWidget extends JPanel {
    static final long serialVersionUID = -8642646142406127052L;

    /** An array of labels representing the places of the map, in
     * [x][y] order. */
    protected JLabel places[][];

    /** The dimensions of the widget (not necessarily of the map). */
    protected int width, height;

    /**
     * Create a new MapWidget.
     *
     * @param plan a string representation of the map with which to
     * initialize the widget; this representation determines the
     * dimensions of the widget.
     */
    public MapWidget(String plan) {
        java.util.List<String> lines
            = new java.util.LinkedList<String>(Arrays.asList(plan.split("\n")));
        lines.removeAll(Collections.singleton(""));

        if (lines.size() == 0)
            width = height = 0;
        else {
            height = lines.size();
            width = lines.get(0).length();
        }

        setBackground(Color.BLACK);
        setForeground(Color.GRAY);

        setLayout(new GridLayout(height, width));

        places = new JLabel[width][height];

        for (int y = 0; y < height; ++y) {
            String line = lines.get(y);

            for (int x = 0; x < width; ++x) {
                add(places[x][y] = new JLabel("" + line.charAt(x),
                                              JLabel.CENTER));
                places[x][y].setFont(new Font(Font.MONOSPACED, Font.BOLD, 36));
            }
        }
    }

    /**
     * Update the widget without drawing a player.
     */
    public void update(String plan) { update(plan, false); }

    /**
     * Update the widget.
     *
     * @param plan a string representation of the map to be drawn.  If
     * this is smaller than the widget's dimensions, it will be
     * centred within the widget.
     * @param showPlayer whether or not to override the centre space
     * of the plan and instead show a player.
     */
    public void update(String plan, boolean showPlayer) {
        int height, width;

        java.util.List<String> lines
            = new java.util.LinkedList<String>(Arrays.asList(plan.split("\n")));
        lines.removeAll(Collections.singleton(""));

        if (lines.size() == 0)
            width = height = 0;
        else {
            height = lines.size();
            width = lines.get(0).length();
        }

        int xOffset = (this.width - width) / 2,
            yOffset = (this.height - height) / 2;

        for (int y = 0; y < this.height; ++y)
            for (int x = 0; x < this.width; ++x)
                if (y < yOffset || x < xOffset
                    || y >= height + yOffset || x >= width + xOffset)
                    places[x][y].setText("X");
                else if (showPlayer && y == height / 2 + 1
                                    && x == width / 2 + 1)
                    places[x][y].setText("P");
                else
                    places[x][y].setText("" + lines.get(y - yOffset)
                                         .charAt(x - xOffset));
    }
}