import uk.co.twey.uni.dooom.*;

import org.junit.*;
import java.io.*;

public class MapTest {
    @Test(expected = MapFormatException.class)
    public void testErrorEmpty() throws MapFormatException {
        new MapOfDooom("Empty map", 1, "");
    }

    @Test
    public void testMiniMap() throws MapFormatException {
        MapOfDooom map = new MapOfDooom("Tiny map", 0, "#E \nG. ");
        Assert.assertTrue(map.getPlace(0, 0) instanceof WallOfDooom);
        Assert.assertTrue(map.getPlace(1, 0) instanceof ExitOfDooom);
        Assert.assertTrue(map.getPlace(2, 0) instanceof DarknessOfDooom);
        Assert.assertTrue(map.getPlace(0, 1) instanceof FloorOfDooom);
        Assert.assertTrue(map.getPlace(1, 1) instanceof FloorOfDooom);
        Assert.assertEquals(map.getGoal(), 0);
        Assert.assertEquals(map.getName(), "Tiny map");
        Assert.assertEquals(3, map.getFloor().size());
    }

    @Test(expected = MapFormatException.class)
    public void testErrorUnknownPlace() throws MapFormatException {
        new MapOfDooom("Unknown map", 1, "?");
    }

    @Test(expected = MapFormatException.class)
    public void testErrorIrregularMap() throws MapFormatException {
        new MapOfDooom("Irregular map", 1, "...\n..");
    }

    @Test
    public void testFileLoad() throws MapFormatException, IOException {
        MapOfDooom.fromFile(new File("test/data/square.map"));
    }

    @Test(expected = MapFormatException.class)
    public void testErrorNonNumGoal() throws MapFormatException, IOException {
        MapOfDooom.fromFile(new File("test/data/nonnum.map"));
    }

    @Test(expected = MapFormatException.class)
    public void testErrorNonIntGoal() throws MapFormatException, IOException {
        MapOfDooom.fromFile(new File("test/data/nonint.map"));
    }

    @Test(expected = MapFormatException.class)
    public void testErrorNoGoal() throws MapFormatException, IOException {
        MapOfDooom.fromFile(new File("test/data/nogoal.map"));
    }

    @Test(expected = MapFormatException.class)
    public void testErrorNegGoal() throws MapFormatException, IOException {
        MapOfDooom.fromFile(new File("test/data/neg.map"));
    }

    @Test(expected = MapFormatException.class)
    public void testErrorUnnamed() throws MapFormatException, IOException {
        MapOfDooom.fromFile(new File("test/data/unnamed.map"));
    }
}